package SPVM::Native::MethodCall::Callback;



1;

=head1 Name

SPVM::Native::MethodCall::Callback - Short Description

=head1 Description

The Native::MethodCall::Callback interface of L<SPVM> has interface methods for someting.

=head1 Usage

  interface Native::MethodCall::Callback;

=head1 Interface Methods




=head1 Repository



=head1 Author

Yuki Kimoto C<kimoto.yuki@gmail.com>

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

